@@ -1,4 +1,6 @@
+use crate::hash::NodeHash;
 use crate::hash::{hash_node, ERROR_HASH};
+use crate::tally::TallyList;
 use crate::utilstring::to_hex;
 use crate::{Validation, VoteReference};
 #[cfg(feature = "with-serde")]
@@ -19,6 +21,36 @@ pub struct Node {
     pub left: NodeRef,
     /// Second child node
     pub right: NodeRef,
+
+    #[cfg_attr(feature = "with-serde", serde(skip_serializing))]
+    /// (Optional) cache of the node hash
+    pub hash: Option<(NodeHash, Option<TallyList>)>,
+}
+
+impl Node {
+    /// Create a new tally tree node.
+    ///
+    /// If `use_cache` is true, the nodes will
+    /// be hashed as they are created. This is slower, but improves
+    /// performance on calls that require many hash operations, such as
+    /// creating proofs.
+    pub fn new(
+        vote: Option<(VoteReference, Vec<u32>)>,
+        left_child: NodeRef,
+        right_child: NodeRef,
+        use_cache: bool,
+    ) -> Result<Node, String> {
+        let mut n = Node {
+            vote,
+            left: left_child,
+            right: right_child,
+            hash: None,
+        };
+        if use_cache {
+            n.hash = Some(hash_node(&n, &Validation::Relaxed)?);
+        }
+        Ok(n)
+    }
 }
 
 fn pretty_format_node(node: &Node, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -59,7 +91,7 @@ impl fmt::Debug for Node {
 /// use tallytree::generate::generate_tree;
 /// let tree = generate_tree(vec![
 ///     ([0xaa; 32], vec![1, 0]),
-/// ]).unwrap();
+/// ], false).unwrap();
 /// assert!(is_leaf_node(&tree.unwrap().left.as_ref().unwrap().left))
 /// ```
 pub fn is_leaf_node(node: &NodeRef) -> bool {
@@ -78,7 +110,7 @@ pub fn is_leaf_node(node: &NodeRef) -> bool {
 /// use tallytree::generate::generate_tree;
 /// let tree = generate_tree(vec![
 ///     ([0xaa; 32], vec![1, 0]),
-/// ]).unwrap();
+/// ], false).unwrap();
 /// assert!(is_wrapper_node(&tree.unwrap().left))
 /// ```
 pub fn is_wrapper_node(node: &NodeRef) -> bool {
@@ -97,7 +129,7 @@ pub fn is_wrapper_node(node: &NodeRef) -> bool {
 /// use tallytree::generate::generate_tree;
 /// let tree = generate_tree(vec![
 ///     ([0xaa; 32], vec![1, 0]),
-/// ]).unwrap();
+/// ], false).unwrap();
 /// assert!(is_null_node_ref(&tree.unwrap().right))
 /// ```
 pub fn is_null_node_ref(node: &NodeRef) -> bool {
@@ -131,6 +163,7 @@ pub fn null_node() -> Node {
         vote: None,
         left: None,
         right: None,
+        hash: None,
     }
 }
 
