@@ -69,7 +69,7 @@ pub type ExclusionProof = Option<(Option<Proof>, Option<Proof>)>;
 /// let tree = generate_tree(vec![
 ///   ([0xaa; 32], vec![1, 0]),
 ///   ([0xcc; 32], vec![1, 0]),
-/// ]).unwrap();
+/// ], true).unwrap();
 /// let (inclusion, _) = create_inclusion_exclusion_proof(&tree, &[0xaa; 32], &Validation::Strict).unwrap();
 /// let (_, exclusion) = create_inclusion_exclusion_proof(&tree, &[0xbb; 32], &Validation::Strict).unwrap();
 /// ```
@@ -118,7 +118,7 @@ pub fn create_inclusion_exclusion_proof(
 /// let tree = generate_tree(vec![
 ///   ([0xaa; 32], vec![1, 0]),
 ///   ([0xcc; 32], vec![1, 0]),
-/// ]).unwrap();
+/// ], true).unwrap();
 /// let v = &Validation::Strict;
 /// let (inclusion, _) = create_inclusion_exclusion_proof(&tree, &[0xaa; 32], v).unwrap();
 /// let (merkle_tree_hash, _, _) = verify_inclusion_proof(&inclusion, &[0xaa; 32], v).unwrap();
@@ -182,7 +182,7 @@ pub fn verify_inclusion_proof(
 /// let tree = generate_tree(vec![
 ///   ([0xaa; 32], vec![1, 0]),
 ///   ([0xcc; 32], vec![1, 0]),
-/// ]).unwrap();
+/// ], true).unwrap();
 /// let v = &Validation::Strict;
 /// let (_, exclusion) = create_inclusion_exclusion_proof(&tree, &[0xbb; 32], v).unwrap();
 /// let (merkle_tree_hash, _) = verify_exclusion_proof(&exclusion, &[0xbb; 32], v).unwrap();
@@ -319,6 +319,10 @@ mod tests {
     use super::*;
     use crate::generate::generate_tree;
     use crate::hash::hash_node_ref;
+    #[cfg(feature = "with-benchmarks")]
+    use crate::utiltest::*;
+    #[cfg(feature = "with-benchmarks")]
+    use test::Bencher;
 
     #[test]
     fn test_create_inclusion_proof() {
@@ -329,11 +333,14 @@ mod tests {
         //       D   E  F  Ø
         //       |   |  |
         //       V1  V2 V3
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xdd; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xdd; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
         let (inclusion, exclusion) =
             create_inclusion_exclusion_proof(&root, &[0xaa; 32], &Validation::Strict).unwrap();
@@ -360,11 +367,14 @@ mod tests {
         //       D   E  F  Ø
         //       |   |  |
         //       V1  V2 V3
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xdd; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xdd; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
         // Voter 0xcc does not exist. Should return merkle tree to 0xaa and 0xdd to prove this.
         let (inclusion, exclusion) =
@@ -389,11 +399,14 @@ mod tests {
         //       D   E  F  Ø
         //       |   |  |
         //       V1  V2 V3
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xdd; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xdd; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
 
         let v = &Validation::Strict;
@@ -419,11 +432,14 @@ mod tests {
         //       D   E  F  Ø
         //       |   |  |
         //       V1  V2 V3
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xdd; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xdd; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
 
         let v = &Validation::Strict;
@@ -450,11 +466,14 @@ mod tests {
         //       D   E  F  Ø
         //       |   |  |
         //       V1  V2 V3
-        let root = generate_tree(vec![
-            ([0xbb; 32], vec![1, 0]),
-            ([0xcc; 32], vec![1, 0]),
-            ([0xee; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xbb; 32], vec![1, 0]),
+                ([0xcc; 32], vec![1, 0]),
+                ([0xee; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
         let v = &Validation::Strict;
         let merkle_hash_and_tally = hash_node_ref(&root, v).unwrap().unwrap();
@@ -479,7 +498,11 @@ mod tests {
     #[test]
     fn test_verify_valid_exclusion_proof_2() {
         // Test case with only two votes.
-        let tree = generate_tree(vec![([0xaa; 32], vec![1, 0]), ([0xcc; 32], vec![1, 0])]).unwrap();
+        let tree = generate_tree(
+            vec![([0xaa; 32], vec![1, 0]), ([0xcc; 32], vec![1, 0])],
+            true,
+        )
+        .unwrap();
         let v = &Validation::Strict;
         let (_, exclusion) = create_inclusion_exclusion_proof(&tree, &[0xbb; 32], v).unwrap();
         let (merkle_tree_hash, _) = verify_exclusion_proof(&exclusion, &[0xbb; 32], v).unwrap();
@@ -495,4 +518,120 @@ mod tests {
         let err = verify_exclusion_proof(&Some((None, None)), &[0xdd; 32], v);
         assert_eq!(err, Err("Invalid proof (Both trees are None)".to_string()))
     }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_create_inclusion_proof_1k_nocache(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(1000);
+        let tree = generate_tree(votes, false).unwrap();
+        let vote: VoteReference = dummy_hash_from_number(42);
+        assert!(
+            create_inclusion_exclusion_proof(&tree, &vote, &Validation::Relaxed)
+                .unwrap()
+                .0
+                .is_some()
+        );
+        b.iter(|| create_inclusion_exclusion_proof(&tree, &vote, &Validation::Relaxed).unwrap())
+    }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_create_inclusion_proof_1k_cache(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(1000);
+        let tree = generate_tree(votes, true).unwrap();
+        let vote: VoteReference = dummy_hash_from_number(42);
+        assert!(
+            create_inclusion_exclusion_proof(&tree, &vote, &Validation::Relaxed)
+                .unwrap()
+                .0
+                .is_some()
+        );
+        b.iter(|| create_inclusion_exclusion_proof(&tree, &vote, &Validation::Relaxed).unwrap())
+    }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_create_inclusion_proof_5k_nocache(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(5000);
+        let tree = generate_tree(votes, false).unwrap();
+        let vote: VoteReference = dummy_hash_from_number(42);
+        assert!(
+            create_inclusion_exclusion_proof(&tree, &vote, &Validation::Strict)
+                .unwrap()
+                .0
+                .is_some()
+        );
+        b.iter(|| create_inclusion_exclusion_proof(&tree, &vote, &Validation::Strict).unwrap())
+    }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_create_inclusion_proof_5k_cache(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(5000);
+        let tree = generate_tree(votes, true).unwrap();
+        let vote: VoteReference = dummy_hash_from_number(42);
+        assert!(
+            create_inclusion_exclusion_proof(&tree, &vote, &Validation::Strict)
+                .unwrap()
+                .0
+                .is_some()
+        );
+        b.iter(|| create_inclusion_exclusion_proof(&tree, &vote, &Validation::Strict).unwrap())
+    }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_create_exclusion_proof_1k_nocache(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(1000);
+        let tree = generate_tree(votes, false).unwrap();
+        let excluded_vote: VoteReference = dummy_hash_from_number(43);
+        assert!(
+            create_inclusion_exclusion_proof(&tree, &excluded_vote, &Validation::Strict)
+                .unwrap()
+                .1
+                .is_some()
+        );
+        b.iter(|| {
+            create_inclusion_exclusion_proof(&tree, &excluded_vote, &Validation::Strict).unwrap()
+        })
+    }
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_create_exclusion_proof_1k_cache(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(1000);
+        let tree = generate_tree(votes, true).unwrap();
+        let excluded_vote: VoteReference = dummy_hash_from_number(43);
+        assert!(
+            create_inclusion_exclusion_proof(&tree, &excluded_vote, &Validation::Strict)
+                .unwrap()
+                .1
+                .is_some()
+        );
+        b.iter(|| {
+            create_inclusion_exclusion_proof(&tree, &excluded_vote, &Validation::Strict).unwrap()
+        })
+    }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_verify_inclusion_proof_100k(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(100_000);
+        let tree = generate_tree(votes, true).unwrap();
+        let vote: VoteReference = dummy_hash_from_number(42);
+        let proof = create_inclusion_exclusion_proof(&tree, &vote, &Validation::Strict).unwrap();
+
+        b.iter(|| verify_inclusion_proof(&proof.0, &vote, &Validation::Strict).unwrap())
+    }
+
+    #[cfg(feature = "with-benchmarks")]
+    #[bench]
+    fn bench_verify_exclusion_proof_100k(b: &mut Bencher) {
+        let votes = crate::utiltest::create_votes(100_000);
+        let tree = generate_tree(votes, true).unwrap();
+        let excluded_vote: VoteReference = dummy_hash_from_number(43);
+        let proof =
+            create_inclusion_exclusion_proof(&tree, &excluded_vote, &Validation::Strict).unwrap();
+
+        b.iter(|| verify_exclusion_proof(&proof.1, &excluded_vote, &Validation::Strict).unwrap())
+    }
 }
