@@ -69,7 +69,7 @@ pub fn is_vote_in_proof(vote: &VoteReference, proof: &Proof) -> bool {
  *      (a_vote, vec![1, 0]),
  *      (b_vote, vec![1, 0]),
  *      (c_vote, vec![0, 1]),
- * ]).unwrap();
+ * ], true).unwrap();
  *
  * let v = &Validation::Strict;
  * let a_proof = create_merkle_proof(&find_node_by_vote_reference(&tree, &a_vote).unwrap(), v).unwrap();
@@ -168,7 +168,7 @@ pub fn are_neighboors(
 ///      ([0xaa; 32], vec![1, 0]),
 ///      ([0xbb; 32], vec![1, 0]),
 ///      ([0xcc; 32], vec![0, 1]),
-/// ]).unwrap();
+/// ], true).unwrap();
 ///
 /// let v = &Validation::Strict;
 /// let proof = create_merkle_proof(
@@ -326,7 +326,7 @@ pub fn derive_wrapper_branch(branch: &ProofBranch, v: &Validation) -> Result<Pro
 /// let (vote_reference, vote) = ([0xaa; 32], vec![1, 0]);
 /// let tree = generate_tree(vec![
 ///     (vote_reference, vote.clone()),
-/// ]).unwrap();
+/// ], false).unwrap();
 /// let wrapper_hash = derive_wrapper_hash(&hash_leaf(&vote_reference, &vote), &vote);
 /// assert_eq!(&wrapper_hash,
 ///     hash_node_ref(&tree.unwrap().as_ref().left, v).unwrap().unwrap().0.as_array());
@@ -364,7 +364,7 @@ pub fn derive_wrapper_hash(hash: &NodeHash, tally: &[u32]) -> ProofHash {
 ///      ([0xaa; 32], vec![1, 0]),
 ///      ([0xbb; 32], vec![1, 0]),
 ///      ([0xcc; 32], vec![0, 1]),
-/// ]).unwrap();
+/// ], true).unwrap();
 ///
 /// let v = &Validation::Strict;
 /// let proof = create_merkle_proof(
@@ -401,7 +401,7 @@ pub fn directions_to_node_index(directions: &[TraverseDirection]) -> usize {
 ///      ([0xaa; 32], vec![1, 0]),
 ///      ([0xbb; 32], vec![1, 0]),
 ///      ([0xcc; 32], vec![0, 1]),
-/// ]).unwrap();
+/// ], true).unwrap();
 ///
 /// let proof = create_merkle_proof(
 ///     &find_node_by_vote_reference(&tree, &[0xcc; 32]).unwrap(),
@@ -479,7 +479,7 @@ pub fn create_merkle_proof(path: &[NodeRef], v: &Validation) -> Result<Proof, St
 ///      ([0xaa; 32], vec![1, 0]),
 ///      ([0xbb; 32], vec![1, 0]),
 ///      ([0xcc; 32], vec![0, 1]),
-/// ]).unwrap();
+/// ], true).unwrap();
 ///
 /// let v = &Validation::Strict;
 /// let proof = create_merkle_proof(
@@ -556,11 +556,14 @@ mod tests {
         let b_vote: VoteReference = [0xbb; 32];
         let c_vote: VoteReference = [0xcc; 32];
 
-        let root = generate_tree(vec![
-            (a_vote, vec![1, 0]),
-            (b_vote, vec![1, 0]),
-            (c_vote, vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                (a_vote, vec![1, 0]),
+                (b_vote, vec![1, 0]),
+                (c_vote, vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
 
         let v = &Validation::Strict;
@@ -593,11 +596,14 @@ mod tests {
         //       |   |  |
         //       V1  V2 V3
         // ```
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xcc; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xcc; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
         let a = root.clone();
         let b = a.as_ref().unwrap().left.clone();
@@ -636,13 +642,16 @@ mod tests {
         // ```
         //
         // Collapsing the merkle proof for V5, Ø should give directions "Right, Left"
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![0, 1]),
-            ([0xcc; 32], vec![0, 1]),
-            ([0xdd; 32], vec![1, 0]),
-            ([0xee; 32], vec![1, 0]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![0, 1]),
+                ([0xcc; 32], vec![0, 1]),
+                ([0xdd; 32], vec![1, 0]),
+                ([0xee; 32], vec![1, 0]),
+            ],
+            true,
+        )
         .unwrap();
         let proof = create_proof_of_vote_count(&root, &Validation::Strict).unwrap();
 
@@ -665,13 +674,16 @@ mod tests {
 
     #[test]
     fn test_directions_to_node_index() {
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![0, 1]),
-            ([0xcc; 32], vec![0, 1]),
-            ([0xdd; 32], vec![1, 0]),
-            ([0xee; 32], vec![1, 0]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![0, 1]),
+                ([0xcc; 32], vec![0, 1]),
+                ([0xdd; 32], vec![1, 0]),
+                ([0xee; 32], vec![1, 0]),
+            ],
+            true,
+        )
         .unwrap();
         let proof = create_proof_of_vote_count(&root, &Validation::Strict).unwrap();
 
@@ -697,11 +709,14 @@ mod tests {
         //       V1  V2 V3
         // ```
 
-        let tree = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xcc; 32], vec![0, 1]),
-        ])
+        let tree = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xcc; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
         let v = &Validation::Strict;
         let root_hash_and_tally = hash_node_ref(&tree, v).unwrap().unwrap();
@@ -824,11 +839,14 @@ mod tests {
         //       |   |  |
         //       V1  V2 V3
         // ```
-        let root = generate_tree(vec![
-            ([0xaa; 32], vec![1, 0]),
-            ([0xbb; 32], vec![1, 0]),
-            ([0xcc; 32], vec![0, 1]),
-        ])
+        let root = generate_tree(
+            vec![
+                ([0xaa; 32], vec![1, 0]),
+                ([0xbb; 32], vec![1, 0]),
+                ([0xcc; 32], vec![0, 1]),
+            ],
+            true,
+        )
         .unwrap();
         let v = &Validation::Strict;
         let a_proof =
